#include <Arduino.h>
#include "lorae5.h"
#include "config_application.h"
#include "config_board.h"

uint8_t sizePayloadUp = 4;
uint8_t sizePayloadDown = 0;

uint8_t payloadUp[20] = {0x00, 0x01, 0x02, 0x03};
uint8_t payloadDown[20]  ={0};

LORAE5 lorae5(devEUI, appEUI, appKey, devAddr, nwkSKey, appSKey);

/***********************************************************************/
/* Please see README page on https://github.com/SylvainMontagny/LoRaE5 */
/***********************************************************************/

// Code de GPT
#define RXD2 16  // Broche RX de l'ESP32 (connectée au TX du LoRa-E5)
#define TXD2 17  // Broche TX de l'ESP32 (connectée au RX du LoRa-E5)
HardwareSerial SerialAT(2); // Utilisation de Serial2 pour LoRa-E5



void setup() {
  lorae5.setup_hardware(&Debug_Serial, &LoRa_Serial);
  lorae5.setup_lorawan(REGION, ACTIVATION_MODE, CLASS, SPREADING_FACTOR, ADAPTIVE_DR, CONFIRMED, PORT_UP, SEND_BY_PUSH_BUTTON, FRAME_DELAY);
  lorae5.printInfo();
  Debug_Serial.begin(115200);
  LoRa_Serial.begin(9600, SERIAL_8N1, RXD2, TXD2);


  if(ACTIVATION_MODE == OTAA){
    Debug_Serial.println("Join Procedure in progress...");
    while(lorae5.join() == false);
    delay(2000);
   }
}
void processDownlink() {
  // You have received "sizePayloadDown" bytes stored in the table "payloadDown"
}


void loop() {
  // change of data : 
  // 🔥 Simulation de capteur : température entre 20 et 30°C, humidité entre 40 et 80%
  uint8_t temperature = random(20, 30);  // Température entre 20 et 30°C
  uint8_t humidite = random(40, 80);     // Humidité entre 40% et 80%

  payloadUp[1] = temperature;
  payloadUp[2] = humidite;



  // Send data
  lorae5.sendData(payloadUp, sizePayloadUp);

  // Check downlink on RX1-RX2 and call processDownlink() 
  if (lorae5.awaitForDownlinkClass_A(payloadDown, &sizePayloadDown) == RET_DOWNLINK){
    processDownlink();
  };

  // Wait in Low Power mode
  lorae5.sleep();
}

