#include <Arduino.h>
#include <HardwareSerial.h>
#include "SdsDustSensor.h"
#include "esp_sleep.h"
#include <Preferences.h>

#define RX_PIN 19
#define TX_PIN 18
#define LED_PIN 2  // LED intégrée sur la plupart des ESP32

HardwareSerial SDS_Serial(1);
SdsDustSensor sds(SDS_Serial);
Preferences prefs;

void clignoteLED(int nbFois, int duree) {
    for (int i = 0; i < nbFois; i++) {
        digitalWrite(LED_PIN, HIGH);
        delay(duree);
        digitalWrite(LED_PIN, LOW);
        delay(duree);
    }
}

void setup() {
    Serial.begin(115200);
    delay(500);

    pinMode(LED_PIN, OUTPUT);
    digitalWrite(LED_PIN, LOW);

    // === Initialisation du capteur ===
    SDS_Serial.begin(9600, SERIAL_8N1, RX_PIN, TX_PIN);
    sds.begin();
    sds.wakeup();

    digitalWrite(LED_PIN, HIGH);  // LED allumée pendant stabilisation
    delay(25000);

    // === Lecture d'une mesure ===
    PmResult pm = sds.readPm();
    float pm25 = -1, pm10 = -1;

    if (pm.isOk()) {
        pm25 = pm.pm25;
        pm10 = pm.pm10;
        Serial.printf("📊 Mesure obtenue : PM2.5 = %.1f µg/m³, PM10 = %.1f µg/m³\n", pm25, pm10);
        clignoteLED(3, 300);  // 3 clignotements lents = mesure OK
    } else {
        Serial.println("❌ Erreur de lecture du capteur SDS011.");
        clignoteLED(3, 100);  // 3 clignotements rapides = erreur
    }

    sds.sleep();

    // === Stockage en mémoire NVS ===
    prefs.begin("mesures", false);
    int cycle = prefs.getInt("cycle", 0);
    prefs.putFloat(("pm25_" + String(cycle)).c_str(), pm25);
    prefs.putFloat(("pm10_" + String(cycle)).c_str(), pm10);
    prefs.putInt("cycle", cycle + 1);
    prefs.end();

    Serial.printf("✅ Mesure stockée en NVS [cycle %d]\n", cycle + 1);

    // === Si 5 cycles atteints ===
    if (cycle + 1 >= 5) {
        Serial.println("\n📥 Récupération des 5 mesures stockées :");

        prefs.begin("mesures", false);
        for (int i = 0; i < 5; i++) {
            float val_pm25 = prefs.getFloat(("pm25_" + String(i)).c_str(), -1);
            float val_pm10 = prefs.getFloat(("pm10_" + String(i)).c_str(), -1);
            Serial.printf("📝 Mesure %d : PM2.5 = %.1f, PM10 = %.1f\n", i + 1, val_pm25, val_pm10);
        }

        prefs.clear();  // nettoyage pour redémarrer à 0
        prefs.end();

        clignoteLED(5, 200);  // signal de fin de série
    }

    digitalWrite(LED_PIN, LOW);  // LED éteinte avant de dormir

    Serial.println("😴 Mise en veille de l’ESP32 pour 5 secondes...");
    Serial.flush();
    esp_sleep_enable_timer_wakeup(5 * 1000000);
    esp_deep_sleep_start();
}

void loop() {
    // Vide
}
