#include <Arduino.h>
#include <HardwareSerial.h>
#include "SdsDustSensor.h"

// Définition des broches UART pour le SDS011
#define RX_PIN 19  // GPIO19 pour RX
#define TX_PIN 18  // GPIO18 pour TX

// Création du port série UART1 pour le capteur SDS011
HardwareSerial SDS_Serial(1);

// Création de l'objet SDS011 avec le bon HardwareSerial
SdsDustSensor sds(SDS_Serial);

void setup() {
    Serial.begin(115200);  // Moniteur série USB

    // Initialisation du port série pour le SDS011
    SDS_Serial.begin(9600, SERIAL_8N1, RX_PIN, TX_PIN);
    
    // Initialisation du capteur SDS011
    sds.begin();

    Serial.println("SDS011 Initialisé !");
    Serial.println(sds.queryFirmwareVersion().toString());
    Serial.println(sds.setActiveReportingMode().toString());
    Serial.println(sds.setContinuousWorkingPeriod().toString()); 
}

void loop() {

    PmResult pm = sds.readPm();
    if (pm.isOk()) {
        Serial.print("PM2.5 = ");
        Serial.print(pm.pm25);
        Serial.print(" µg/m³, PM10 = ");
        Serial.print(pm.pm10);
        Serial.println(" µg/m³");
        Serial.println(pm.toString());
    } else {
        Serial.print("Erreur de lecture SDS011 : ");
        Serial.println(pm.statusToString());
    }

    delay(1000);
}
