#include <Arduino.h>
#include <SoftwareSerial.h>
#include "SdsDustSensor.h"

// Définition des broches pour le SDS011 (sur ESP8266)
#define RX_PIN 5  // GPIO14 = D5 (RX de l'ESP, donc relié au TX du SDS011)
#define TX_PIN 4  // GPIO12 = D6 (TX de l'ESP, donc relié au RX du SDS011)

// Création du port série logiciel pour le capteur SDS011
SoftwareSerial SDS_Serial(RX_PIN, TX_PIN);  // RX, TX

// Objet SDS011
SdsDustSensor sds(SDS_Serial);

void setup() {
    Serial.begin(115200);  // Moniteur série USB

    SDS_Serial.begin(9600);
    sds.begin();

    Serial.println("SDS011 Initialisé !");
    Serial.println(sds.queryFirmwareVersion().toString());
    Serial.println(sds.setActiveReportingMode().toString());
    Serial.println(sds.setContinuousWorkingPeriod().toString()); 
}

void loop() {
    PmResult pm = sds.readPm();
    if (pm.isOk()) {
        Serial.print("PM2.5 = ");
        Serial.print(pm.pm25);
        Serial.print(" µg/m³, PM10 = ");
        Serial.print(pm.pm10);
        Serial.println(" µg/m³");
        Serial.println(pm.toString());
    } else {
        Serial.print("Erreur de lecture SDS011 : ");
        Serial.println(pm.statusToString());
    }

    delay(1000);
}
