#include <Arduino.h>
#include <SoftwareSerial.h>                     // 👈 LIGNE 1 — Changement de bibliothèque
#include "SdsDustSensor.h"

// Définition des broches UART pour le SDS011
#define RX_PIN D5  // GPIO14                  // 👈 LIGNE 2 — Changement de pin
#define TX_PIN D6  // GPIO12                  // 👈 LIGNE 3 — Changement de pin

// Création d’un port série logiciel
SoftwareSerial SDS_Serial(RX_PIN, TX_PIN);     // 👈 LIGNE 4 — Utilisation de SoftwareSerial

// Création de l'objet SDS011 avec le bon SoftwareSerial
SdsDustSensor sds(SDS_Serial);

void setup() {
    Serial.begin(115200);  // Moniteur série USB

    // Initialisation du port série pour le SDS011
    SDS_Serial.begin(9600);                       // 👈 LIGNE 5 — Simplifiée pour SoftwareSerial
    
    // Initialisation du capteur SDS011
    sds.begin();

    Serial.println("SDS011 Initialisé !");
    Serial.println(sds.queryFirmwareVersion().toString());
    Serial.println(sds.setActiveReportingMode().toString());
    Serial.println(sds.setContinuousWorkingPeriod().toString()); 
}

void loop() {
    Serial.println("Réveil du SDS011...");
    sds.wakeup();
    delay(10000); // Attente initiale (10s) pour stabilisation minimale

    // 🔁 Faire 3 mesures toutes les 10 secondes
    for (int i = 0; i < 3; i++) {
        PmResult pm = sds.readPm();
        if (pm.isOk()) {
            Serial.print("Mesure ");
            Serial.print(i + 1);
            Serial.print(" - PM2.5 = ");
            Serial.print(pm.pm25);
            Serial.print(" µg/m³, PM10 = ");
            Serial.print(pm.pm10);
            Serial.println(" µg/m³");
        } else {
            Serial.print("Erreur mesure ");
            Serial.print(i + 1);
            Serial.print(" : ");
            Serial.println(pm.statusToString());
        }

        delay(10000); // ⏳ Pause de 10 secondes entre chaque mesure
    }

    Serial.println("Mise en veille du SDS011...");
    sds.sleep();

    delay(30000); // 💤 Attente 30 secondes en mode sommeil
}
