#include <Arduino.h>
#include "config_board.h"
#include <HardwareSerial.h>
#include "SdsDustSensor.h"

// Définition des broches UART pour le SDS011
#define RX_PIN_SDS 19
#define TX_PIN_SDS 18

// Définition des broches UART pour le LoRa-E5
#define RXD2 16
#define TXD2 17

// Déclaration des objets Serial
HardwareSerial SDS_Serial(1);  // UART1 pour SDS011
HardwareSerial LoRaSerial(2);  // UART2 pour LoRa-E5

// Buffer pour envoyer
uint8_t payloadUp[4];

void sendCommand(String cmd) {
    LoRaSerial.println(cmd);
    delay(100);
    while (LoRaSerial.available()) {
        Serial.write(LoRaSerial.read()); // Pour debug
    }
}

void setup() {
    Serial.begin(115200);
    delay(2000);

    // Init UART pour SDS011
    SDS_Serial.begin(9600, SERIAL_8N1, RX_PIN_SDS, TX_PIN_SDS);
    SdsDustSensor sds(SDS_Serial);
    sds.begin();
    Serial.println("SDS011 Initialisé !");
    Serial.println(sds.queryFirmwareVersion().toString());
    Serial.println(sds.setActiveReportingMode().toString());
    Serial.println(sds.setContinuousWorkingPeriod().toString());

    // Init UART pour LoRa-E5
    LoRaSerial.begin(9600, SERIAL_8N1, RXD2, TXD2);
    delay(500);

    Serial.println("Mise en mode LoRa P2P");

    // Configurer le LoRa-E5 pour du LoRa point à point
    sendCommand("AT+MODE=TEST");  // Basculer en mode test (P2P)
    delay(500);
    sendCommand("AT+TEST=RFCFG,868100000,20,9,7,8");  
    // fréquence 868.1 MHz, puissance 20 dBm, SF9, BW 125 kHz, CR 4/8

    delay(1000);
}

void loop() {
    static SdsDustSensor sds(SDS_Serial);
    PmResult pm = sds.readPm();

    if (pm.isOk()) {
        Serial.print("PM2.5 = ");
        Serial.print(pm.pm25);
        Serial.print(" µg/m³, PM10 = ");
        Serial.print(pm.pm10);
        Serial.println(" µg/m³");

        uint16_t pm25 = static_cast<uint16_t>(pm.pm25 * 10); 
        uint16_t pm10 = static_cast<uint16_t>(pm.pm10 * 10);

        payloadUp[0] = (pm25 >> 8) & 0xFF;
        payloadUp[1] = pm25 & 0xFF;
        payloadUp[2] = (pm10 >> 8) & 0xFF;
        payloadUp[3] = pm10 & 0xFF;

        Serial.print("Payload envoyé (HEX) : ");
        for (int i = 0; i < sizeof(payloadUp); i++) {
            Serial.print("0x");
            Serial.print(payloadUp[i], HEX);
            Serial.print(" ");
        }
        Serial.println();

        // Convertir payload en String hexadécimal
        String payloadStr = "";
        for (int i = 0; i < sizeof(payloadUp); i++) {
            if (payloadUp[i] < 16) payloadStr += "0"; // ajouter 0 devant si nécessaire
            payloadStr += String(payloadUp[i], HEX);
        }
        payloadStr.toUpperCase(); // Majuscule pour respecter AT+TEST syntaxe

        // Envoi via LoRa
        sendCommand("AT+TEST=TXLRPKT," + payloadStr);

        delay(3000);
    } else {
        Serial.print("Erreur de lecture SDS011 : ");
        Serial.println(pm.statusToString());
    }
}
