#include <Arduino.h>
#include "lorae5.h"
#include "config_application.h"
#include "config_board.h"
#include <HardwareSerial.h>
#include "SdsDustSensor.h"

// Définition des broches UART pour le SDS011
#define RX_PIN_SDS 19  // GPIO19 pour RX du SDS011
#define TX_PIN_SDS 18  // GPIO18 pour TX du SDS011

// Définition des broches UART pour le LoRa-E5
#define RXD2 16  // GPIO16 pour RX du LoRa-E5
#define TXD2 17  // GPIO17 pour TX du LoRa-E5

// Déclaration des objets Serial
HardwareSerial SDS_Serial(1);  // UART1 pour SDS011

// Création des objets capteur et LoRa
SdsDustSensor sds(SDS_Serial);
LORAE5 lorae5(devEUI, appEUI, appKey, devAddr, nwkSKey, appSKey);

// Buffer pour l'envoi des données LPP (2 channels x 4 octets)
uint8_t payloadUp[8];

void setup() {
    Serial.begin(115200);
    
    // Initialisation du port série pour le SDS011
    SDS_Serial.begin(9600, SERIAL_8N1, RX_PIN_SDS, TX_PIN_SDS);
    sds.begin();
    Serial.println("SDS011 Initialisé !");
    Serial.println(sds.queryFirmwareVersion().toString());
    Serial.println(sds.setActiveReportingMode().toString());
    Serial.println(sds.setContinuousWorkingPeriod().toString()); 

    // Initialisation du module LoRa-E5
    lorae5.setup_hardware(&Debug_Serial, &Serial2);
    lorae5.setup_lorawan(REGION, ACTIVATION_MODE, CLASS, SPREADING_FACTOR, ADAPTIVE_DR, CONFIRMED, PORT_UP, SEND_BY_PUSH_BUTTON, FRAME_DELAY);
    lorae5.printInfo();
    
    Debug_Serial.begin(115200);
    Serial2.begin(9600, SERIAL_8N1, RXD2, TXD2);

    // Connexion OTAA si activée
    if (ACTIVATION_MODE == OTAA) {
        Debug_Serial.println("Join Procedure in progress...");
        while (!lorae5.join());
        delay(2000);
    }
}

void loop() {
    PmResult pm = sds.readPm();

    if (pm.isOk()) {
        Serial.print("PM2.5 = ");
        Serial.print(pm.pm25);
        Serial.print(" µg/m³, PM10 = ");
        Serial.print(pm.pm10);
        Serial.println(" µg/m³");

        // Conversion avec 1 chiffre après la virgule (x10)
        uint16_t pm25 = static_cast<uint16_t>(pm.pm25 * 100);
        uint16_t pm10 = static_cast<uint16_t>(pm.pm10 * 100);

        // Encodage Cayenne LPP
        // Channel 1 - PM2.5
        payloadUp[0] = 0x01;               // Channel
        payloadUp[1] = 0x02;               // Analog Input
        payloadUp[2] = (pm25 >> 8) & 0xFF; // MSB
        payloadUp[3] = pm25 & 0xFF;        // LSB

        // Channel 2 - PM10
        payloadUp[4] = 0x02;               // Channel
        payloadUp[5] = 0x02;               // Analog Input
        payloadUp[6] = (pm10 >> 8) & 0xFF; // MSB
        payloadUp[7] = pm10 & 0xFF;        // LSB

        // Debug Payload
        Serial.print("Payload LPP envoyé (HEX) : ");
        for (int i = 0; i < sizeof(payloadUp); i++) {
            Serial.print("0x");
            if (payloadUp[i] < 16) Serial.print("0");
            Serial.print(payloadUp[i], HEX);
            Serial.print(" ");
        }
        Serial.println();

        // Envoi LoRa
        lorae5.sendData(payloadUp, sizeof(payloadUp));

        delay(30000); // Attente 30s avant la prochaine mesure
    } else {
        Serial.print("Erreur de lecture SDS011 : ");
        Serial.println(pm.statusToString());
    }

    // Mise en veille
    lorae5.sleep();
}
