#include <Arduino.h>
#include "lorae5.h"
#include "config_application.h"
#include "config_board.h"
#include <HardwareSerial.h>
#include "SdsDustSensor.h"
#include "DHT.h"



// Définition des broches UART pour le SDS011
#define RX_PIN_SDS 19  // GPIO19 pour RX du SDS011
#define TX_PIN_SDS 18  // GPIO18 pour TX du SDS011

// Définition des broches UART pour le LoRa-E5
#define RXD2 16  // GPIO16 pour RX du LoRa-E5
#define TXD2 17  // GPIO17 pour TX du LoRa-E5

// Définition des broches pour le DHT22
#define DHTPIN 22     // GPIO22 pour le DHT22
#define DHTTYPE DHT22   // DHT 22  (AM2302), AM2321


DHT dht(DHTPIN, DHTTYPE);





// Déclaration des objets Serial
HardwareSerial SDS_Serial(1);  // UART1 pour SDS011

// Création des objets capteur et LoRa
SdsDustSensor sds(SDS_Serial);
LORAE5 lorae5(devEUI, appEUI, appKey, devAddr, nwkSKey, appSKey);

// Buffer pour l'envoi des données (4 octets, avec uint8_t pour bien gérer les octets)
uint8_t payloadUp[8];

void setup() {
    Serial.begin(115200);
    // Initialisation du capteur DHT
    dht.begin();
    Serial.println(F("Capteur DHTxx initialisé."));


    // Initialisation du port série pour le SDS011
    SDS_Serial.begin(9600, SERIAL_8N1, RX_PIN_SDS, TX_PIN_SDS);
    sds.begin();
    Serial.println("SDS011 Initialisé !");
    Serial.println(sds.queryFirmwareVersion().toString());
    Serial.println(sds.setActiveReportingMode().toString());
    Serial.println(sds.setContinuousWorkingPeriod().toString()); 

    // Initialisation du module LoRa-E5
    lorae5.setup_hardware(&Debug_Serial, &Serial2);
    lorae5.setup_lorawan(REGION, ACTIVATION_MODE, CLASS, SPREADING_FACTOR, ADAPTIVE_DR, CONFIRMED, PORT_UP, SEND_BY_PUSH_BUTTON, FRAME_DELAY);
    lorae5.printInfo();
    
    Debug_Serial.begin(115200);
    Serial2.begin(9600, SERIAL_8N1, RXD2, TXD2);

    // Connexion OTAA si activée
    if (ACTIVATION_MODE == OTAA) {
        Debug_Serial.println("Join Procedure in progress...");
        while (!lorae5.join());
        delay(2000);
    }
}

void loop() {
    PmResult pm = sds.readPm();

    //On imprime les données sur le moniteur série
    if (pm.isOk()) {
        Serial.print("PM2.5 = ");
        Serial.print(pm.pm25);
        Serial.print(" µg/m³, PM10 = ");
        Serial.print(pm.pm10);
        Serial.println(" µg/m³");
        // Reading temperature or humidity takes about 250 milliseconds!
        // Sensor readings may also be up to 2 seconds 'old' (its a very slow sensor)
        float h = dht.readHumidity();
        // Read temperature as Celsius (the default)
        float t = dht.readTemperature();

        // Check if any reads failed and exit early (to try again).
        if (isnan(h) || isnan(t)) {
            Serial.println(F("Failed to read from DHT sensor!"));
            return;
        }
        Serial.print(F("Humidity: "));
        Serial.print(h);
        Serial.print(F("%  Temperature: "));
        Serial.print(t);
        Serial.print(F("°C "));



        // Conversion en uint16_t avec *10 pour garder 1 chiffre après la virgule
        uint16_t pm25 = static_cast<uint16_t>(pm.pm25 * 10);  // Valeurs de PM2.5
        uint16_t pm10 = static_cast<uint16_t>(pm.pm10 * 10);  // Valeurs de PM10
        uint16_t temp = static_cast<uint16_t>(t * 10); // Température
        uint16_t hum  = static_cast<uint16_t>(h * 10); // Humidité

        // Stockage dans le payload en respectant l'ordre Little-Endian
        payloadUp[0] = (pm25 >> 8) & 0xFF; // Octet fort en premier ✅
        payloadUp[1] = pm25 & 0xFF;        // Octet faible en second ✅
        payloadUp[2] = (pm10 >> 8) & 0xFF; // Octet fort en premier ✅
        payloadUp[3] = pm10 & 0xFF;        // Octet faible en second ✅

        // Code pour la température et l'humidité
        payloadUp[4] = (temp >> 8) & 0xFF;
        payloadUp[5] = temp & 0xFF;
        payloadUp[6] = (hum >> 8) & 0xFF;
        payloadUp[7] = hum & 0xFF;

        // Affichage du payload avant envoi (debug)
        Serial.print("Payload envoyé (HEX) : ");
        for (int i = 0; i < sizeof(payloadUp); i++) {
            Serial.print("0x");
            Serial.print(payloadUp[i], HEX);
            Serial.print(" ");
        }
        Serial.println();

        // Envoi via LoRa
        lorae5.sendData(payloadUp, sizeof(payloadUp));

        // Pause avant le prochain envoi (modifiable)
        delay(30000);
    } else {
        Serial.print("Erreur de lecture SDS011 : ");
        Serial.println(pm.statusToString());
    }

    // Mise en veille pour économie d'énergie
    lorae5.sleep();
}
