#include <Arduino.h>
#include <Wire.h>
#include <SoftwareSerial.h>
#include "lorae5.h"
#include "config_application.h"
#include "config_board.h"
#include "SdsDustSensor.h"
#include <DFRobot_SHT3x.h>

// SDS011 SoftwareSerial pins (ESP8266)
#define SDS_RX 5  // D1 = GPIO5
#define SDS_TX 4  // D2 = GPIO4

// LoRa-E5 SoftwareSerial pins (ESP8266)
#define LORA_RX 13 // D7 = GPIO13
#define LORA_TX 2  // TX = GPIO1 (⚠️ port USB partagé avec Serial)

SoftwareSerial SDS_Serial(SDS_RX, SDS_TX);
SoftwareSerial LoRaSerial(LORA_RX, LORA_TX);

SdsDustSensor sds(SDS_Serial);
DFRobot_SHT3x sht3x;
LORAE5 lorae5(devEUI, appEUI, appKey, devAddr, nwkSKey, appSKey);

uint8_t payloadUp[8];

void setup() {
  Serial.begin(115200);
  Wire.begin(12, 14);  // SDA = GPIO12 (D6), SCL = GPIO14 (D5)

  // Init SHT30
  while (sht3x.begin() != 0) {
    Serial.println("❌ Erreur initialisation SHT30");
    delay(1000);
  }
  if (!sht3x.startPeriodicMode(sht3x.eMeasureFreq_1Hz)) {
    Serial.println("⚠️ SHT30: échec mode périodique");
  }

  // Init SDS011
  SDS_Serial.begin(9600);
  sds.begin();
  sds.wakeup();
  // Init LoRa
  LoRaSerial.begin(9600);  // ✅ Initialisé ici, plus besoin dans lorae5.cpp
  delay(300);
  lorae5.setup_hardware(&Serial, &LoRaSerial);  // OK car LoRaSerial hérite de Stream
  lorae5.setup_lorawan(REGION, ACTIVATION_MODE, CLASS, SPREADING_FACTOR, ADAPTIVE_DR, CONFIRMED, PORT_UP, SEND_BY_PUSH_BUTTON, FRAME_DELAY);
  Serial.println("Pour le moment ca marche");

  if (ACTIVATION_MODE == OTAA) {
    Serial.println("DEBUG: Appel à lorae5.join()...");

    while (!lorae5.join()) {
      Serial.println("⏳ Tentative de join...");
      delay(2000);
    }
  }
  Serial.println("Pour le moment ca marche_2");

}

void loop() {
  PmResult pm = sds.readPm();
  if (!pm.isOk()) {
    Serial.print("❌ SDS011: ");
    Serial.println(pm.statusToString());
    delay(3000);
    return;
  }

  delay(1100);  // Recommandé pour SHT30
  auto data = sht3x.readTemperatureAndHumidity();
  if (data.ERR != 0) {
    Serial.println("❌ Erreur lecture SHT30");
    return;
  }

  float t = data.TemperatureC;
  float h = data.Humidity;
  Serial.printf("PM2.5: %.1f | PM10: %.1f | Temp: %.1f°C | Hum: %.1f%%\n", pm.pm25, pm.pm10, t, h);

  // Encodage HEX sur 2 octets chacun
  uint16_t pm25 = pm.pm25 * 10;
  uint16_t pm10 = pm.pm10 * 10;
  uint16_t temp = t * 10;
  uint16_t hum = h * 10;

  payloadUp[0] = pm25 >> 8;
  payloadUp[1] = pm25 & 0xFF;
  payloadUp[2] = pm10 >> 8;
  payloadUp[3] = pm10 & 0xFF;
  payloadUp[4] = temp >> 8;
  payloadUp[5] = temp & 0xFF;
  payloadUp[6] = hum >> 8;
  payloadUp[7] = hum & 0xFF;

  Serial.print("Payload: ");
  for (uint8_t i = 0; i < 8; i++) {
    Serial.printf("%02X ", payloadUp[i]);
  }
  Serial.println();

  lorae5.sendData(payloadUp, sizeof(payloadUp));
  delay(30000);
  lorae5.sleep();
}
