#include <Arduino.h>
#include "SdsDustSensor.h"
#include <Wire.h>
#include <DFRobot_SHT3x.h>
#include <stdarg.h>

// SDS011
#define RX_PIN_SDS 19
#define TX_PIN_SDS 18

// LoRa-E5 UART
#define RXD2 16
#define TXD2 17

HardwareSerial SDS_Serial(1);
SdsDustSensor sds(SDS_Serial);
DFRobot_SHT3x sht3x;

static char recv_buf[512];
int counter = 0;

// Fonction AT
static int at_send_check_response(const char *p_ack, int timeout_ms, const char *p_cmd, ...)
{
  va_list args;
  int index = 0;
  int ch;
  int startMillis;
  memset(recv_buf, 0, sizeof(recv_buf));
  va_start(args, p_cmd);
  char cmd[256];
  vsnprintf(cmd, sizeof(cmd), p_cmd, args);
  va_end(args);
  Serial2.print(cmd);
  Serial.print("-> "); Serial.print(cmd);
  delay(200);
  startMillis = millis();

  if (p_ack == NULL) return 0;

  do {
    while (Serial2.available()) {
      ch = Serial2.read();
      if (index < sizeof(recv_buf) - 1) {
        recv_buf[index++] = ch;
      }
      Serial.print((char)ch);
    }
    if (strstr(recv_buf, p_ack)) return 1;
  } while (millis() - startMillis < timeout_ms);
  return 0;
}

void setup() {
  Serial.begin(115200);
  Wire.begin(23, 22); // SHT30 : SDA=23, SCL=22

  // SHT30 init
  while (sht3x.begin() != 0) {
    Serial.println("Erreur SHT30");
    delay(1000);
  }
  sht3x.startPeriodicMode(sht3x.eMeasureFreq_1Hz);

  // SDS011 init
  SDS_Serial.begin(9600, SERIAL_8N1, RX_PIN_SDS, TX_PIN_SDS);
  sds.begin();

  // LoRa AT init
  Serial2.begin(9600, SERIAL_8N1, RXD2, TXD2);
  delay(100);
  at_send_check_response("+AT: OK", 100, "AT\r\n");
  at_send_check_response("+MODE: TEST", 1000, "AT+MODE=TEST\r\n");
  at_send_check_response("+TEST: TXLRPKT", 5000, "AT+TEST=TXLRPKT,\"HELLO\"\r\n"); // test initial
}

void loop() {
  PmResult pm = sds.readPm();
  if (!pm.isOk()) {
    Serial.println("Erreur SDS011 !");
    delay(5000);
    return;
  }

  delay(1100); // SHT30 timing
  DFRobot_SHT3x::sRHAndTemp_t data = sht3x.readTemperatureAndHumidity();
  if (data.ERR != 0) {
    Serial.println("Erreur lecture SHT30 !");
    delay(5000);
    return;
  }

  // Conversion en uint16
  uint16_t pm25 = pm.pm25 * 10;
  uint16_t pm10 = pm.pm10 * 10;
  uint16_t temp = data.TemperatureC * 10;
  uint16_t hum  = data.Humidity * 10;

  // Construction payload en hexadécimal
  char hexPayload[32];
  sprintf(hexPayload, "%04X%04X%04X%04X", pm25, pm10, temp, hum);

  char cmd[64];
  sprintf(cmd, "AT+TEST=TXLRPKT,\"%s\"\r\n", hexPayload);

  int ok = at_send_check_response("+TEST: TXLRPKT", 3000, cmd);
  if (ok)
    Serial.println("Données envoyées !");
  else
    Serial.println("Échec d'envoi.");

  delay(30000); // 30 s
}
