#include <Arduino.h>
#include "lorae5.h"
#include "config_application.h"
#include "config_board.h"
#include <HardwareSerial.h>
#include "SdsDustSensor.h"
#include <Wire.h>
#include <DFRobot_SHT3x.h>

// Broches UART pour le SDS011
#define RX_PIN_SDS 19
#define TX_PIN_SDS 18

// Broches UART pour le LoRa-E5
#define RXD2 16
#define TXD2 17

// Création des objets
HardwareSerial SDS_Serial(1);
SdsDustSensor sds(SDS_Serial);
LORAE5 lorae5(devEUI, appEUI, appKey, devAddr, nwkSKey, appSKey);
DFRobot_SHT3x sht3x;

// Buffer LoRa
uint8_t payloadUp[8];

void setup() {
    Serial.begin(115200);
    Wire.begin(23, 22);  // SDA = 23, SCL = 22

    // Init SHT30
    while (sht3x.begin() != 0) {
        Serial.println("Erreur d'initialisation SHT30 !");
        delay(1000);
    }
    if (!sht3x.startPeriodicMode(sht3x.eMeasureFreq_1Hz)) {
        Serial.println("Échec du passage en mode périodique SHT30");
    }

    // Init SDS011
    SDS_Serial.begin(9600, SERIAL_8N1, RX_PIN_SDS, TX_PIN_SDS);
    sds.begin();
    Serial.println(sds.queryFirmwareVersion().toString());

    // Init LoRa
    Debug_Serial.begin(115200);
    Serial2.begin(9600, SERIAL_8N1, RXD2, TXD2);
    lorae5.setup_hardware(&Debug_Serial, &Serial2);
    lorae5.setup_lorawan(REGION, ACTIVATION_MODE, CLASS, SPREADING_FACTOR, ADAPTIVE_DR, CONFIRMED, PORT_UP, SEND_BY_PUSH_BUTTON, FRAME_DELAY);
    if (ACTIVATION_MODE == OTAA) {
        while (!lorae5.join());
        delay(2000);
    }
}

void loop() {
    PmResult pm = sds.readPm();
    if (pm.isOk()) {
        // Attendre que le capteur ait eu le temps de prendre une mesure
        delay(1100);

        // Lecture regroupée de température et humidité
        DFRobot_SHT3x::sRHAndTemp_t data = sht3x.readTemperatureAndHumidity();
        if (data.ERR != 0) {
            Serial.println("Erreur de lecture SHT30");
            return;
        }

        float t = data.TemperatureC;
        float h = data.Humidity;

        Serial.print("PM2.5: "); Serial.print(pm.pm25);
        Serial.print(" | PM10: "); Serial.print(pm.pm10);
        Serial.print(" | Temp: "); Serial.print(t);
        Serial.print(" | Hum: "); Serial.println(h);

        // Conversion
        uint16_t pm25 = static_cast<uint16_t>(pm.pm25 * 10);
        uint16_t pm10 = static_cast<uint16_t>(pm.pm10 * 10);
        uint16_t temp = static_cast<uint16_t>(t * 10);
        uint16_t hum  = static_cast<uint16_t>(h * 10);

        // Encodage
        payloadUp[0] = (pm25 >> 8) & 0xFF;
        payloadUp[1] = pm25 & 0xFF;
        payloadUp[2] = (pm10 >> 8) & 0xFF;
        payloadUp[3] = pm10 & 0xFF;
        payloadUp[4] = (temp >> 8) & 0xFF;
        payloadUp[5] = temp & 0xFF;
        payloadUp[6] = (hum >> 8) & 0xFF;
        payloadUp[7] = hum & 0xFF;

        // Affichage
        Serial.print("Payload HEX: ");
        for (int i = 0; i < 8; i++) {
            Serial.print("0x"); Serial.print(payloadUp[i], HEX); Serial.print(" ");
        }
        Serial.println();

        lorae5.sendData(payloadUp, sizeof(payloadUp));
        delay(30000);
    } else {
        Serial.print("Erreur SDS011 : ");
        Serial.println(pm.statusToString());
    }

    lorae5.sleep();
}
